@@ -1,5 +1,5 @@
 /*
- * $Id: pamc_converse.c,v 1.2 2001/01/20 22:29:47 agmorgan Exp $
+ * $Id: pamc_converse.c,v 1.3 2005/05/20 14:58:58 kukuk Exp $
  *
  * Copyright (c) Andrew G. Morgan <morgan@ftp.kernel.org>
  *
@@ -34,8 +34,8 @@ static int __pamc_select_agent(pamc_handle_t pch, char *agent_id)
 
 int pamc_converse(pamc_handle_t pch, pamc_bp_t *prompt_p)
 {
-    __u32 size, offset=0;
-    __u8 control, raw[PAM_BP_MIN_SIZE];
+    u_int32_t size, offset=0;
+    u_int8_t control, raw[PAM_BP_MIN_SIZE];
 
     D(("called"));
 
@@ -109,7 +109,7 @@ int pamc_converse(pamc_handle_t pch, pamc_bp_t *prompt_p)
     /* pump all of the prompt into the agent */
     do {
 	int rval = write(pch->current->writer,
-			 offset + (const __u8 *) (*prompt_p),
+			 offset + (const u_int8_t *) (*prompt_p),
 			 size - offset);
 	if (rval == -1) {
 	    switch (errno) {
@@ -171,7 +171,7 @@ int pamc_converse(pamc_handle_t pch, pamc_bp_t *prompt_p)
        value from the previous loop */
 
     while (offset < size) {
-	int rval = read(pch->current->reader, offset + (__u8 *) *prompt_p,
+	int rval = read(pch->current->reader, offset + (u_int8_t *) *prompt_p,
 			size-offset);
 
 	if (rval == -1) {
