@@ -1,13 +1,13 @@
 /* pam_group module */
 
 /*
- * $Id: pam_group.c,v 1.7 2004/09/24 13:13:20 kukuk Exp $
+ * $Id: pam_group.c,v 1.10 2005/06/09 17:29:19 kukuk Exp $
  *
  * Written by Andrew Morgan <morgan@linux.kernel.org> 1996/7/6
  */
 
-const static char rcsid[] =
-"$Id: pam_group.c,v 1.7 2004/09/24 13:13:20 kukuk Exp $;\n"
+static const char rcsid[] =
+"$Id: pam_group.c,v 1.10 2005/06/09 17:29:19 kukuk Exp $;\n"
 "Version 0.5 for Linux-PAM\n"
 "Copyright (c) Andrew G. Morgan 1996 <morgan@linux.kernel.org>\n";
 
@@ -36,11 +36,11 @@ const static char rcsid[] =
 #define PAM_GROUP_BUFLEN        1000
 #define FIELD_SEPARATOR         ';'   /* this is new as of .02 */
 
-#ifdef TRUE 
-# undef TRUE 
-#endif 
-#ifdef FALSE 
-# undef FALSE 
+#ifdef TRUE
+# undef TRUE
+#endif
+#ifdef FALSE
+# undef FALSE
 #endif
 
 typedef enum { FALSE, TRUE } boolean;
@@ -140,7 +140,7 @@ static int read_field(int fd, char **buf, int *from, int *to)
 	    fd = -1;          /* end of file reached */
 	} else
 	    *to += i;
-    
+
 	/*
 	 * contract the buffer. Delete any comments, and replace all
 	 * multiple spaces with single commas
@@ -630,7 +630,11 @@ static int check_account(pam_handle_t *pamh, const char *service,
     if (no_grps > 0) {
 	grps = calloc( blk_size(no_grps) , sizeof(gid_t) );
 	D(("copying current list into grps [%d big]",blk_size(no_grps)));
-	(void) getgroups(no_grps, grps);
+	if (getgroups(no_grps, grps) < 0) {
+	    D(("getgroups call failed"));
+	    no_grps = 0;
+	    grps = NULL;
+	}
 #ifdef DEBUG
 	{
 	    int z;
@@ -772,8 +776,9 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags
 PAM_EXTERN int pam_sm_setcred(pam_handle_t *pamh, int flags
 			      , int argc, const char **argv)
 {
-    const char *service=NULL, *tty=NULL;
+    const void *service=NULL, *void_tty=NULL;
     const char *user=NULL;
+    const char *tty;
     int retval;
     unsigned setting;
 
@@ -787,7 +792,7 @@ PAM_EXTERN int pam_sm_setcred(pam_handle_t *pamh, int flags
 
     /* set service name */
 
-    if (pam_get_item(pamh, PAM_SERVICE, (const void **)&service)
+    if (pam_get_item(pamh, PAM_SERVICE, &service)
 	!= PAM_SUCCESS || service == NULL) {
 	_log_err("cannot find the current service name");
 	return PAM_ABORT;
@@ -803,8 +808,8 @@ PAM_EXTERN int pam_sm_setcred(pam_handle_t *pamh, int flags
 
     /* set tty name */
 
-    if (pam_get_item(pamh, PAM_TTY, (const void **)&tty) != PAM_SUCCESS
-	|| tty == NULL) {
+    if (pam_get_item(pamh, PAM_TTY, &void_tty) != PAM_SUCCESS
+	|| void_tty == NULL) {
 	D(("PAM_TTY not set, probing stdin"));
 	tty = ttyname(STDIN_FILENO);
 	if (tty == NULL) {
@@ -816,6 +821,8 @@ PAM_EXTERN int pam_sm_setcred(pam_handle_t *pamh, int flags
 	    return PAM_ABORT;
 	}
     }
+    else
+      tty = (const char *) void_tty;
 
     if (strncmp("/dev/",tty,5) == 0) {          /* strip leading /dev/ */
 	tty += 5;
