@@ -50,7 +50,7 @@ int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc,
     FILE *fd;
     int parse_esc = 1;
     char *prompt_tmp = NULL;
-    const char *cur_prompt = NULL;
+    const void *cur_prompt = NULL;
     struct stat st;
     char *issue_file = NULL;
 
@@ -91,8 +91,7 @@ int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc,
 	    return PAM_IGNORE;
 	}
 
-	retval = pam_get_item(pamh, PAM_USER_PROMPT,
-			      (const void **) &cur_prompt);
+	retval = pam_get_item(pamh, PAM_USER_PROMPT, &cur_prompt);
 	if (retval != PAM_SUCCESS) {
 	    fclose(fd);
 	    if (issue_file)
@@ -219,8 +218,10 @@ static char *do_prompt(FILE *fd)
 		{
 		    char domainname[256];
 
-		    getdomainname(domainname, sizeof(domainname));
-		    domainname[sizeof(domainname)-1] = '\0';
+		    if (getdomainname(domainname, sizeof(domainname)) == -1)
+		      domainname[0] = '\0';
+		    else
+		      domainname[sizeof(domainname)-1] = '\0';
 		    snprintf (buf, 1024, "%s", domainname);
 		}
 		break;
@@ -244,7 +245,7 @@ static char *do_prompt(FILE *fd)
 		    if (c == 'd')
 			snprintf (buf, 1024, "%s %s %d  %d",
 				weekday[tm->tm_wday], month[tm->tm_mon],
-				tm->tm_mday, 
+				tm->tm_mday,
 				tm->tm_year + 1900);
 		    else
 			snprintf (buf, 1024, "%02d:%02d:%02d",
@@ -269,10 +270,11 @@ static char *do_prompt(FILE *fd)
 			if (ut->ut_type == USER_PROCESS)
 			users++;
 		    endutent();
-		    printf ("%d ", users);
 		    if (c == 'U')
-			snprintf (buf, 1024, "%s", (users == 1) ?
-				" user" : " users");
+			snprintf (buf, 1024, "%d %s", users,
+			          (users == 1) ? "user" : "users");
+		    else
+			snprintf (buf, 1024, "%d", users);
 		    break;
 		}
 	      default:
