@@ -1,7 +1,7 @@
 /* pam_nologin module */
 
 /*
- * $Id: pam_nologin.c,v 1.5 2004/09/22 09:37:49 kukuk Exp $
+ * $Id: pam_nologin.c,v 1.7 2005/06/09 17:29:19 kukuk Exp $
  *
  * Written by Michael K. Johnson <johnsonm@redhat.com> 1996/10/24
  *
@@ -80,7 +80,8 @@ static int perform_check(pam_handle_t *pamh, struct opt_s *opts)
 
 	char *mtmp=NULL;
 	struct passwd *user_pwd;
-	struct pam_conv *conversation;
+	const void *void_conv;
+	const struct pam_conv *conversation;
 	struct pam_message message;
 	struct pam_message *pmessage = &message;
 	struct pam_response *resp = NULL;
@@ -121,14 +122,15 @@ static int perform_check(pam_handle_t *pamh, struct opt_s *opts)
 		mtmp[st.st_size] = '\000';
 
 		/*
-		 * Use conversation function to give user contents 
+		 * Use conversation function to give user contents
 		 * of /etc/nologin
 		 */
 
-		retval = pam_get_item(pamh, PAM_CONV, 
-				(const void **)&conversation);
-		if ((retval == PAM_SUCCESS) && (conversation)) {
-			(void) conversation->conv(1, 
+		if (pam_get_item(pamh, PAM_CONV, &void_conv)
+		    == PAM_SUCCESS && void_conv &&
+		    ((const struct pam_conv *)void_conv)->conv) {
+		        conversation = void_conv;
+			(void) conversation->conv(1,
 				(const struct pam_message **)&pmessage,
 				&resp, conversation->appdata_ptr);
 
