@@ -42,7 +42,7 @@
 
 #define USER_RHOSTS_FILE "/.rhosts"     /* prefixed by user's home dir */
 
-#ifdef linux
+#ifdef __linux__
 #include <endian.h>
 #endif
 
@@ -50,6 +50,10 @@
 #include <sys/fsuid.h>
 #endif /* HAVE_SYS_FSUID_H */
 
+#ifdef HAVE_NET_IF_H
+#include <sys/if.h>
+#endif
+
 #include <sys/types.h>
 #include <sys/uio.h>
 #include <string.h>
@@ -74,19 +78,13 @@ int innetgr(const char *, const char *, const char *,const char *);
 #include <ctype.h>
 
 #include <net/if.h>
-#ifdef linux
-# include <linux/sockios.h>
-# ifndef __USE_MISC
-#  define __USE_MISC
-#  include <sys/fsuid.h>
-# endif /* __USE_MISC */
-#endif
 
 #include <pwd.h>
 #include <grp.h>
 #include <sys/file.h>
 #include <sys/signal.h>
 #include <sys/stat.h>
+#include <stdint.h>
 #include <syslog.h>
 #ifndef _PATH_HEQUIV
 #define _PATH_HEQUIV "/etc/hosts.equiv"
@@ -98,9 +96,13 @@ int innetgr(const char *, const char *, const char *,const char *);
 #include <security/_pam_macros.h>
 #include <security/_pam_modutil.h>
 
-/* to the best of my knowledge, all modern UNIX boxes have 32 bit integers */
-#define U32 unsigned int
-
+#ifdef HAVE_INTTYPES_H
+  #include <inttypes.h>
+  #define U32 uint32_t
+#else
+  /* to the best of my knowledge, all modern UNIX boxes have 32 bits integers */
+  #define U32 unsigned int
+#endif /* HAVE_INTTYPES_H */
 
 /*
  * Options for this module
@@ -183,7 +185,7 @@ static void set_option (struct _options *opts, const char *arg)
 	return;
     }
 
-    if (strcmp(arg, "superuser=") == 0) {
+    if (strncmp(arg, "superuser=", sizeof("superuser=")-1) == 0) {
 	opts->superuser = arg+sizeof("superuser=")-1;
 	return;
     }
@@ -214,9 +216,9 @@ static int pam_get_rhost(pam_handle_t *pamh, const char **rhost
 			 , const char *prompt)
 {
     int retval;
-    const char   *current;
+    const void *current;
 
-    retval = pam_get_item (pamh, PAM_RHOST, (const void **)&current);
+    retval = pam_get_item (pamh, PAM_RHOST, &current);
     if (retval != PAM_SUCCESS)
         return retval;
 
@@ -237,9 +239,9 @@ static int pam_get_ruser(pam_handle_t *pamh, const char **ruser,
 			 const char *prompt)
 {
     int retval;
-    const char   *current;
+    const void *current;
 
-    retval = pam_get_item (pamh, PAM_RUSER, (const void **)&current);
+    retval = pam_get_item (pamh, PAM_RUSER, &current);
     if (retval != PAM_SUCCESS) {
         return retval;
     }
@@ -263,7 +265,8 @@ __icheckhost (pam_handle_t *pamh, struct _options *opts, U32 raddr
     struct hostent *hp;
     U32 laddr;
     int negate=1;    /* Multiply return with this to get -1 instead of 1 */
-    char **pp, *user;
+    char **pp;
+    const void *user;
 
     /* Check nis netgroup.  We assume that pam has done all needed
        paranoia checking before we are handed the rhost */
@@ -278,7 +281,7 @@ __icheckhost (pam_handle_t *pamh, struct _options *opts, U32 raddr
 	negate=-1;
 	lhost++;
     } else if (strcmp("+",lhost) == 0) {
-        (void) pam_get_item(pamh, PAM_USER, (const void **)&user);
+        (void) pam_get_item(pamh, PAM_USER, &user);
         D(("user %s has a `+' host entry", user));
 	if (opts->opt_promiscuous)
 	    return (1);                     /* asking for trouble, but ok.. */
@@ -298,7 +301,7 @@ __icheckhost (pam_handle_t *pamh, struct _options *opts, U32 raddr
     hp = gethostbyname(lhost);
     if (hp == NULL)
 	return (0);
-    
+
     /* Spin through ip addresses. */
     for (pp = hp->h_addr_list; *pp; ++pp)
 	if (!memcmp (&raddr, *pp, sizeof (U32)))
@@ -319,7 +322,7 @@ static int __icheckuser(pam_handle_t *pamh, struct _options *opts
       ruser is user id on remote host
       rhost is the remote host name
       */
-    char *user;
+    const void *user;
 
     /* [-+]@netgroup */
     if (strncmp("+@",luser,2) == 0)
@@ -334,8 +337,9 @@ static int __icheckuser(pam_handle_t *pamh, struct _options *opts
 
     /* + */
     if (strcmp("+",luser) == 0) {
-	(void) pam_get_item(pamh, PAM_USER, (const void **)&user);
-	_pam_log(LOG_WARNING, "user %s has a `+' user entry", user);
+	(void) pam_get_item(pamh, PAM_USER, &user);
+	_pam_log(LOG_WARNING, "user %s has a `+' user entry",
+		 (const char *) user);
 	if (opts->opt_promiscuous)
 	    return(1);
 	/* If not promiscuous we handle it as a negative match */
@@ -413,7 +417,7 @@ __ivaliduser (pam_handle_t *pamh, struct _options *opts,
 	    user = p;                   /* this is the user's name */
 	    while (*p && !isspace(*p))
 		++p;                    /* find end of user's name */
-	} else 
+	} else
 	    user = p;
 
 	*p = '\0';              /* <nul> terminate username (+host?) */
@@ -469,7 +473,7 @@ pam_iruserok(pam_handle_t *pamh,
     FILE *hostf;
     uid_t uid;
     int answer;
-    char pbuf[MAXPATHLEN];               /* potential buffer overrun */
+    char *fpath;
 
     if ((!superuser||opts->opt_hosts_equiv_rootok) && !opts->opt_no_hosts_equiv ) {
 
@@ -485,7 +489,7 @@ pam_iruserok(pam_handle_t *pamh,
 	    No hosts.equiv file on system.
 	} */
     }
-    
+
     if ( opts->opt_no_rhosts )
 	return 1;
 
@@ -495,23 +499,22 @@ pam_iruserok(pam_handle_t *pamh,
 
     pwd = _pammodutil_getpwnam(pamh, luser);
     if (pwd == NULL) {
-	/* 
+	/*
 	 * luser is assumed to be valid because of an earlier check for uid = 0
 	 * we don't log this error twice. However, this shouldn't happen !
-	 * --cristiang 
+	 * --cristiang
 	 */
 	return(1);
     }
 
-    /* check for buffer overrun */
-    if (strlen(pwd->pw_dir) + sizeof(USER_RHOSTS_FILE) + 2 >= MAXPATHLEN) {
-	if (opts->opt_debug)
-	    _pam_log(LOG_DEBUG,"home directory for `%s' is too long", luser);
-	return 1;                               /* to dangerous to try */
+    fpath = malloc (strlen (pwd->pw_dir) + strlen (USER_RHOSTS_FILE) + 1);
+    if (fpath == NULL) {
+       _pam_log (LOG_ALERT, "Running out of memory");
+       return 1;
     }
 
-    (void) strcpy(pbuf, pwd->pw_dir);
-    (void) strcat(pbuf, USER_RHOSTS_FILE);
+    strcpy (fpath, pwd->pw_dir);
+    strcat (fpath, USER_RHOSTS_FILE);
 
     /*
      * Change effective uid while _reading_ .rhosts. (not just
@@ -520,19 +523,19 @@ pam_iruserok(pam_handle_t *pamh,
      */
 
     /* We are root, this will not fail */
-#ifdef linux
+#ifdef __linux__
     /* If we are on linux the better way is setfsuid */
     uid = setfsuid(pwd->pw_uid);
-    hostf = fopen(pbuf, "r");
+    hostf = fopen(fpath, "r");
 #else
     uid = geteuid();
     (void) seteuid(pwd->pw_uid);
-    hostf = fopen(pbuf, "r");
+    hostf = fopen(fpath, "r");
 #endif
 
     if (hostf == NULL) {
         if (opts->opt_debug)
-	    _pam_log(LOG_DEBUG,"Could not open %s file",pbuf);
+	    _pam_log(LOG_DEBUG,"Could not open %s file",fpath);
 	answer = 1;
 	goto exit_function;
     }
@@ -543,7 +546,7 @@ pam_iruserok(pam_handle_t *pamh,
      */
 
     cp = NULL;
-    if (lstat(pbuf, &sbuf) < 0 || !S_ISREG(sbuf.st_mode))
+    if (lstat(fpath, &sbuf) < 0 || !S_ISREG(sbuf.st_mode))
 	cp = ".rhosts not regular file";
     else if (fstat(fileno(hostf), &sbuf) < 0)
 	cp = ".rhosts fstat failed";
@@ -555,7 +558,7 @@ pam_iruserok(pam_handle_t *pamh,
 
 	/* private group caveat */
 	if (opts->opt_private_group) {
-	    struct group *grp = getgrgid(sbuf.st_gid);
+	    struct group *grp = _pammodutil_getgrgid(pamh, sbuf.st_gid);
 
 	    if (NULL == grp || NULL == grp->gr_name
 		|| strcmp(luser,grp->gr_name)) {
@@ -596,12 +599,14 @@ exit_function:
      * they are reset before we exit.
      */
 
-#ifdef linux
+#ifdef __linux__
     setfsuid(uid);
 #else
     (void)seteuid(uid);
 #endif
 
+    free (fpath);
+
     if (hostf != NULL)
         (void) fclose(hostf);
 
@@ -657,9 +662,9 @@ pam_ruserok (pam_handle_t *pamh,
  */
 
 static int _pam_auth_rhosts (pam_handle_t *pamh,
-			     int flags, 
+			     int flags,
 			     int argc,
-			     const char **argv) 
+			     const char **argv)
 {
     int retval;
     const char *luser = NULL;
@@ -750,9 +755,9 @@ static int _pam_auth_rhosts (pam_handle_t *pamh,
 /* --- authentication management functions --- */
 
 PAM_EXTERN
-int pam_sm_authenticate (pam_handle_t *pamh, 
+int pam_sm_authenticate (pam_handle_t *pamh,
 			 int flags,
-			 int argc, 
+			 int argc,
 			 const char **argv)
 {
     int retval;
