@@ -1,15 +1,15 @@
 /* pam_time module */
 
 /*
- * $Id: pam_time.c,v 1.4 2001/05/08 05:02:50 agmorgan Exp $
+ * $Id: pam_time.c,v 1.6 2005/06/09 17:29:20 kukuk Exp $
  *
  * Written by Andrew Morgan <morgan@linux.kernel.org> 1996/6/22
  * (File syntax and much other inspiration from the shadow package
  * shadow-960129)
  */
 
-const static char rcsid[] =
-"$Id: pam_time.c,v 1.4 2001/05/08 05:02:50 agmorgan Exp $;\n"
+static const char rcsid[] =
+"$Id: pam_time.c,v 1.6 2005/06/09 17:29:20 kukuk Exp $;\n"
 "\t\tVersion 0.22 for Linux-PAM\n"
 "Copyright (C) Andrew G. Morgan 1996 <morgan@linux.kernel.org>\n";
 
@@ -36,11 +36,11 @@ const static char rcsid[] =
 #define PAM_TIME_BUFLEN        1000
 #define FIELD_SEPARATOR        ';'   /* this is new as of .02 */
 
-#ifdef TRUE 
-# undef TRUE 
-#endif 
-#ifdef FALSE 
-# undef FALSE 
+#ifdef TRUE
+# undef TRUE
+#endif
+#ifdef FALSE
+# undef FALSE
 #endif
 
 typedef enum { FALSE, TRUE } boolean;
@@ -127,13 +127,14 @@ static int read_field(int fd, char **buf, int *from, int *to)
 	i = read(fd, *to + *buf, PAM_TIME_BUFLEN - *to);
 	if (i < 0) {
 	    _log_err("error reading " PAM_TIME_CONF);
+	    close(fd);
 	    return -1;
 	} else if (!i) {
 	    close(fd);
 	    fd = -1;          /* end of file reached */
 	} else
 	    *to += i;
-    
+
 	/*
 	 * contract the buffer. Delete any comments, and replace all
 	 * multiple spaces with single commas
@@ -166,6 +167,7 @@ static int read_field(int fd, char **buf, int *from, int *to)
 		} else {
 		    _log_err("internal error in " __FILE__
 			     " at line %d", __LINE__ );
+		    close(fd);
 		    return -1;
 		}
 		break;
@@ -556,12 +558,13 @@ static int check_account(const char *service
 PAM_EXTERN int pam_sm_acct_mgmt(pam_handle_t *pamh,int flags,int argc
 		     ,const char **argv)
 {
-    const char *service=NULL, *tty=NULL;
+    const void *service=NULL, *void_tty=NULL;
+    const char *tty;
     const char *user=NULL;
 
     /* set service name */
 
-    if (pam_get_item(pamh, PAM_SERVICE, (const void **)&service)
+    if (pam_get_item(pamh, PAM_SERVICE, &service)
 	!= PAM_SUCCESS || service == NULL) {
 	_log_err("cannot find the current service name");
 	return PAM_ABORT;
@@ -577,8 +580,8 @@ PAM_EXTERN int pam_sm_acct_mgmt(pam_handle_t *pamh,int flags,int argc
 
     /* set tty name */
 
-    if (pam_get_item(pamh, PAM_TTY, (const void **)&tty) != PAM_SUCCESS
-	|| tty == NULL) {
+    if (pam_get_item(pamh, PAM_TTY, &void_tty) != PAM_SUCCESS
+	|| void_tty == NULL) {
 	D(("PAM_TTY not set, probing stdin"));
 	tty = ttyname(STDIN_FILENO);
 	if (tty == NULL) {
@@ -590,6 +593,8 @@ PAM_EXTERN int pam_sm_acct_mgmt(pam_handle_t *pamh,int flags,int argc
 	    return PAM_ABORT;
 	}
     }
+    else
+      tty = void_tty;
 
     if (strncmp("/dev/",tty,5) == 0) {          /* strip leading /dev/ */
 	tty += 5;
